@@ -10,21 +10,32 @@ from openpyxl.xml.functions import iterparse
 
 # package imports
 from openpyxl.cell import Cell
+from openpyxl.worksheet.filters import AutoFilter, SortState
 from openpyxl.cell.read_only import _cast_number
-from openpyxl.worksheet import Worksheet, ColumnDimension, RowDimension
+from openpyxl.cell.text import Text
+from openpyxl.worksheet import Worksheet
+from openpyxl.worksheet.dimensions import (
+    ColumnDimension,
+    RowDimension,
+    SheetFormatProperties,
+)
+from openpyxl.worksheet.header_footer import HeaderFooter
+from openpyxl.worksheet.hyperlink import Hyperlink
+from openpyxl.worksheet.merge import MergeCells
 from openpyxl.worksheet.page import PageMargins, PrintOptions, PrintPageSetup
 from openpyxl.worksheet.protection import SheetProtection
-from openpyxl.worksheet.views import SheetView
-from openpyxl.worksheet.datavalidation import DataValidation
+from openpyxl.worksheet.views import SheetViewList
+from openpyxl.worksheet.datavalidation import DataValidationList
 from openpyxl.xml.constants import (
     SHEET_MAIN_NS,
     REL_NS,
     EXT_TYPES,
     PKG_REL_NS
 )
-from openpyxl.xml.functions import safe_iterator
+from openpyxl.xml.functions import safe_iterator, localname
 from openpyxl.styles import Color
-from openpyxl.formatting import ConditionalFormatting, Rule
+from openpyxl.formatting import Rule
+from openpyxl.formatting.formatting import ConditionalFormatting
 from openpyxl.formula.translate import Translator
 from openpyxl.worksheet.properties import WorksheetProperties
 from openpyxl.utils import (
@@ -58,44 +69,49 @@ def _get_xml_iter(xml_source):
 
 class WorkSheetParser(object):
 
-    COL_TAG = '{%s}col' % SHEET_MAIN_NS
-    ROW_TAG = '{%s}row' % SHEET_MAIN_NS
     CELL_TAG = '{%s}c' % SHEET_MAIN_NS
     VALUE_TAG = '{%s}v' % SHEET_MAIN_NS
     FORMULA_TAG = '{%s}f' % SHEET_MAIN_NS
     MERGE_TAG = '{%s}mergeCell' % SHEET_MAIN_NS
-    INLINE_STRING = "{%s}is/{%s}t" % (SHEET_MAIN_NS, SHEET_MAIN_NS)
-    INLINE_RICHTEXT = "{%s}is/{%s}r/{%s}t" % (SHEET_MAIN_NS, SHEET_MAIN_NS, SHEET_MAIN_NS)
+    INLINE_STRING = "{%s}is" % SHEET_MAIN_NS
 
-    def __init__(self, wb, title, xml_source, shared_strings):
-        self.ws = wb.create_sheet(title=title)
+    def __init__(self, ws, xml_source, shared_strings):
+        self.ws = ws
         self.source = xml_source
         self.shared_strings = shared_strings
-        self.guess_types = wb._guess_types
-        self.data_only = wb.data_only
-        self.styles = self.ws.parent._cell_styles
-        self.differential_styles = wb._differential_styles
-        self.keep_vba = wb.vba_archive is not None
+        self.guess_types = ws.parent.guess_types
+        self.data_only = ws.parent.data_only
+        self.styles = ws.parent._cell_styles
+        self.differential_styles = ws.parent._differential_styles
+        self.keep_vba = ws.parent.vba_archive is not None
         self.shared_formula_masters = {}  # {si_str: Translator()}
+        self._row_count = self._col_count = 0
 
     def parse(self):
         dispatcher = {
             '{%s}mergeCells' % SHEET_MAIN_NS: self.parse_merge,
             '{%s}col' % SHEET_MAIN_NS: self.parse_column_dimensions,
-            '{%s}row' % SHEET_MAIN_NS: self.parse_row_dimensions,
-            '{%s}printOptions' % SHEET_MAIN_NS: self.parse_print_options,
-            '{%s}pageMargins' % SHEET_MAIN_NS: self.parse_margins,
-            '{%s}pageSetup' % SHEET_MAIN_NS: self.parse_page_setup,
-            '{%s}headerFooter' % SHEET_MAIN_NS: self.parse_header_footer,
+            '{%s}row' % SHEET_MAIN_NS: self.parse_row,
             '{%s}conditionalFormatting' % SHEET_MAIN_NS: self.parser_conditional_formatting,
-            '{%s}autoFilter' % SHEET_MAIN_NS: self.parse_auto_filter,
-            '{%s}sheetProtection' % SHEET_MAIN_NS: self.parse_sheet_protection,
-            '{%s}dataValidations' % SHEET_MAIN_NS: self.parse_data_validation,
-            '{%s}sheetPr' % SHEET_MAIN_NS: self.parse_properties,
             '{%s}legacyDrawing' % SHEET_MAIN_NS: self.parse_legacy_drawing,
-            '{%s}sheetViews' % SHEET_MAIN_NS: self.parse_sheet_views,
+            '{%s}sheetProtection' % SHEET_MAIN_NS: self.parse_sheet_protection,
             '{%s}extLst' % SHEET_MAIN_NS: self.parse_extensions,
+            '{%s}hyperlink' % SHEET_MAIN_NS: self.parse_hyperlinks,
                       }
+
+        properties = {
+            '{%s}printOptions' % SHEET_MAIN_NS: ('print_options', PrintOptions),
+            '{%s}pageMargins' % SHEET_MAIN_NS: ('page_margins', PageMargins),
+            '{%s}pageSetup' % SHEET_MAIN_NS: ('page_setup', PrintPageSetup),
+            '{%s}headerFooter' % SHEET_MAIN_NS: ('HeaderFooter', HeaderFooter),
+            '{%s}autoFilter' % SHEET_MAIN_NS: ('auto_filter', AutoFilter),
+            '{%s}dataValidations' % SHEET_MAIN_NS: ('data_validations', DataValidationList),
+            '{%s}sortState' % SHEET_MAIN_NS: ('sort_state', SortState),
+            '{%s}sheetPr' % SHEET_MAIN_NS: ('sheet_properties', WorksheetProperties),
+            '{%s}sheetViews' % SHEET_MAIN_NS: ('views', SheetViewList),
+            '{%s}sheetFormatPr' % SHEET_MAIN_NS: ('sheet_format', SheetFormatProperties)
+        }
+
         tags = dispatcher.keys()
         stream = _get_xml_iter(self.source)
         it = iterparse(stream, tag=tags)
@@ -105,9 +121,15 @@ class WorkSheetParser(object):
             if tag_name in dispatcher:
                 dispatcher[tag_name](element)
                 element.clear()
+            elif tag_name in properties:
+                prop = properties[tag_name]
+                obj = prop[1].from_tree(element)
+                setattr(self.ws, prop[0], obj)
+                element.clear()
 
         self.ws._current_row = self.ws.max_row
 
+
     def parse_cell(self, element):
         value = element.find(self.VALUE_TAG)
         if value is not None:
@@ -115,6 +137,7 @@ class WorkSheetParser(object):
         formula = element.find(self.FORMULA_TAG)
         data_type = element.get('t', 'n')
         coordinate = element.get('r')
+        self._col_count += 1
         style_id = element.get('s')
 
         # assign formula to cell value unless only the data is desired
@@ -179,7 +202,11 @@ class WorkSheetParser(object):
             style_id = int(style_id)
             style_array = self.styles[style_id]
 
-        row, column = coordinate_to_tuple(coordinate)
+        if coordinate:
+            row, column = coordinate_to_tuple(coordinate)
+        else:
+            row, column = self._row_count, self._col_count
+
         cell = Cell(self.ws, row=row, col_idx=column, style_array=style_array)
         self.ws._cells[(row, column)] = cell
 
@@ -195,12 +222,11 @@ class WorkSheetParser(object):
 
         else:
             if data_type == 'inlineStr':
-                data_type = 's'
                 child = element.find(self.INLINE_STRING)
-                if child is None:
-                    child = element.find(self.INLINE_RICHTEXT)
                 if child is not None:
-                    value = child.text
+                    data_type = 's'
+                    richtext = Text.from_tree(child)
+                    value = richtext.content
 
         if self.guess_types or value is None:
             cell.value = value
@@ -210,8 +236,10 @@ class WorkSheetParser(object):
 
 
     def parse_merge(self, element):
-        for mergeCell in safe_iterator(element, ('{%s}mergeCell' % SHEET_MAIN_NS)):
-            self.ws.merge_cells(mergeCell.get('ref'))
+        merged = MergeCells.from_tree(element)
+        for c in merged.mergeCell:
+            self.ws.merge_cells(c.ref)
+
 
     def parse_column_dimensions(self, col):
         attrs = dict(col.attrib)
@@ -223,8 +251,14 @@ class WorkSheetParser(object):
         self.ws.column_dimensions[column] = dim
 
 
-    def parse_row_dimensions(self, row):
+    def parse_row(self, row):
         attrs = dict(row.attrib)
+
+        if "r" in attrs:
+            self._row_count = int(attrs['r'])
+        else:
+            self._row_count += 1
+        self._col_count = 0
         keys = set(attrs)
         for key in keys:
             if key == "s":
@@ -243,62 +277,20 @@ class WorkSheetParser(object):
             self.parse_cell(cell)
 
 
-    def parse_print_options(self, element):
-        self.ws.print_options = PrintOptions.from_tree(element)
-
-    def parse_margins(self, element):
-        self.ws.page_margins = PageMargins.from_tree(element)
-
-    def parse_page_setup(self, element):
-        self.ws.page_setup = PrintPageSetup.from_tree(element)
-
-    def parse_header_footer(self, element):
-        oddHeader = element.find('{%s}oddHeader' % SHEET_MAIN_NS)
-        if oddHeader is not None and oddHeader.text is not None:
-            self.ws.header_footer.setHeader(oddHeader.text)
-        oddFooter = element.find('{%s}oddFooter' % SHEET_MAIN_NS)
-        if oddFooter is not None and oddFooter.text is not None:
-            self.ws.header_footer.setFooter(oddFooter.text)
-
-
     def parser_conditional_formatting(self, element):
-        range_string = element.get('sqref')
-        cfRules = element.findall('{%s}cfRule' % SHEET_MAIN_NS)
-        self.ws.conditional_formatting.cf_rules[range_string] = []
-        for node in cfRules:
-            rule = Rule.from_tree(node)
+        cf = ConditionalFormatting.from_tree(element)
+        for rule in cf.rules:
             if rule.dxfId is not None:
                 rule.dxf = self.differential_styles[rule.dxfId]
-            self.ws.conditional_formatting.cf_rules[range_string].append(rule)
+            self.ws.conditional_formatting.add(cf.sqref, rule)
 
 
-    def parse_auto_filter(self, element):
-        self.ws.auto_filter.ref = element.get("ref")
-        for fc in safe_iterator(element, '{%s}filterColumn' % SHEET_MAIN_NS):
-            filters = fc.find('{%s}filters' % SHEET_MAIN_NS)
-            if filters is None:
-                continue
-            vals = [f.get("val") for f in safe_iterator(filters, '{%s}filter' % SHEET_MAIN_NS)]
-            blank = filters.get("blank")
-            self.ws.auto_filter.add_filter_column(fc.get("colId"), vals, blank=blank)
-        for sc in safe_iterator(element, '{%s}sortCondition' % SHEET_MAIN_NS):
-            self.ws.auto_filter.add_sort_condition(sc.get("ref"), sc.get("descending"))
-
     def parse_sheet_protection(self, element):
         self.ws.protection = SheetProtection.from_tree(element)
         password = element.get("password")
         if password is not None:
             self.ws.protection.set_password(password, True)
 
-    def parse_data_validation(self, element):
-        for node in safe_iterator(element, "{%s}dataValidation" % SHEET_MAIN_NS):
-            dv = DataValidation.from_tree(node)
-            self.ws._data_validations.append(dv)
-
-
-    def parse_properties(self, element):
-        self.ws.sheet_properties = WorksheetProperties.from_tree(element)
-
 
     def parse_legacy_drawing(self, element):
         if self.keep_vba:
@@ -307,13 +299,6 @@ class WorkSheetParser(object):
             self.ws.legacy_drawing = element.get('{%s}id' % REL_NS)
 
 
-    def parse_sheet_views(self, element):
-        for el in element.findall("{%s}sheetView" % SHEET_MAIN_NS):
-            # according to the specification the last view wins
-            pass
-        self.ws.sheet_view = SheetView.from_tree(el)
-
-
     def parse_extensions(self, element):
         extLst = ExtensionList.from_tree(element)
         for e in extLst.ext:
@@ -322,7 +307,15 @@ class WorkSheetParser(object):
             warn(msg)
 
 
-def fast_parse(xml_source, parent, sheet_title, shared_strings):
-    parser = WorkSheetParser(parent, sheet_title, xml_source, shared_strings)
-    parser.parse()
-    return parser.ws
+    def parse_hyperlinks(self, element):
+        link = Hyperlink.from_tree(element)
+        if link.id:
+            rel = self.ws._rels[link.id]
+            link.target = rel.Target
+        if ":" in link.ref:
+            # range of cells
+            for row in self.ws[link.ref]:
+                for cell in row:
+                    cell.hyperlink = link
+        else:
+            self.ws[link.ref].hyperlink = link
