@@ -1,54 +1,44 @@
 from __future__ import absolute_import
 # Copyright (c) 2010-2016 openpyxl
 
-""" Iterators-based worksheet reader
-*Still very raw*
+""" Read worksheets on-demand
 """
 
 # compatibility
-from openpyxl.compat import range
+from openpyxl.compat import (
+    range,
+    deprecated
+)
 
 # package
-from openpyxl.compat import removed_method
+from openpyxl.cell.text import Text
+
 from openpyxl.xml.functions import iterparse, safe_iterator
 from openpyxl.xml.constants import SHEET_MAIN_NS
 
 from openpyxl.worksheet import Worksheet
 from openpyxl.utils import (
-    ABSOLUTE_RE,
     column_index_from_string,
     get_column_letter,
     coordinate_to_tuple,
 )
+from openpyxl.worksheet.dimensions import SheetDimension
 from openpyxl.cell.read_only import ReadOnlyCell, EMPTY_CELL
 
 
 def read_dimension(source):
     if hasattr(source, "encode"):
         return
+
     min_row = min_col =  max_row = max_col = None
     DIMENSION_TAG = '{%s}dimension' % SHEET_MAIN_NS
     DATA_TAG = '{%s}sheetData' % SHEET_MAIN_NS
     it = iterparse(source, tag=[DIMENSION_TAG, DATA_TAG])
+
     for _event, element in it:
         if element.tag == DIMENSION_TAG:
-            dim = element.get("ref")
-            m = ABSOLUTE_RE.match(dim.upper())
-            if m is None:
-                return
-            min_col, min_row, sep, max_col, max_row = m.groups()
-            min_row = int(min_row)
-            if max_col is None or max_row is None:
-                max_col = min_col
-                max_row = min_row
-            else:
-                max_row = int(max_row)
-            return (
-                column_index_from_string(min_col),
-                min_row,
-                column_index_from_string(max_col),
-                max_row
-                )
+            dim = SheetDimension.from_tree(element)
+            return dim.boundaries
 
         elif element.tag == DATA_TAG:
             # Dimensions missing
@@ -60,11 +50,11 @@ ROW_TAG = '{%s}row' % SHEET_MAIN_NS
 CELL_TAG = '{%s}c' % SHEET_MAIN_NS
 VALUE_TAG = '{%s}v' % SHEET_MAIN_NS
 FORMULA_TAG = '{%s}f' % SHEET_MAIN_NS
+INLINE_TAG = '{%s}is' % SHEET_MAIN_NS
 DIMENSION_TAG = '{%s}dimension' % SHEET_MAIN_NS
 
-CELL_TAGS = (CELL_TAG, VALUE_TAG, FORMULA_TAG)
 
-class ReadOnlyWorksheet(Worksheet):
+class ReadOnlyWorksheet(object):
 
     _xml = None
     _min_column = 1
@@ -73,8 +63,8 @@ class ReadOnlyWorksheet(Worksheet):
 
     def __init__(self, parent_workbook, title, worksheet_path,
                  xml_source, shared_strings):
-        Worksheet.__init__(self, parent_workbook, title)
-        self._cells = None
+        self.parent = parent_workbook
+        self.title = title
         self._current_row = None
         self.worksheet_path = worksheet_path
         self.shared_strings = shared_strings
@@ -84,6 +74,17 @@ class ReadOnlyWorksheet(Worksheet):
         if dimensions is not None:
             self.min_column, self.min_row, self.max_column, self.max_row = dimensions
 
+        # Methods from Worksheet
+        self.cell = Worksheet.cell.__get__(self)
+        self.iter_rows = Worksheet.iter_rows.__get__(self)
+
+
+    def __getitem__(self, key):
+        # use protected method from Worksheet
+        meth = Worksheet.__getitem__.__get__(self)
+        return meth(key)
+
+
     @property
     def xml_source(self):
         """Parse xml source on demand, default to Excel archive"""
@@ -97,7 +98,12 @@ class ReadOnlyWorksheet(Worksheet):
         self._xml = value
 
 
+    @deprecated("Use ws.iter_rows()")
     def get_squared_range(self, min_col, min_row, max_col, max_row):
+        return self._cells_by_row(min_col, min_row, max_col, max_row)
+
+
+    def _cells_by_row(self, min_col, min_row, max_col, max_row):
         """
         The source worksheet file may have columns or rows missing.
         Missing cells will be created.
@@ -111,7 +117,7 @@ class ReadOnlyWorksheet(Worksheet):
         p = iterparse(self.xml_source, tag=[ROW_TAG], remove_blank_text=True)
         for _event, element in p:
             if element.tag == ROW_TAG:
-                row_id = int(element.get("r"))
+                row_id = int(element.get("r", row_counter))
 
                 # got all the rows we need
                 if max_row is not None and row_id > max_row:
@@ -124,23 +130,23 @@ class ReadOnlyWorksheet(Worksheet):
 
                 # return cells from a row
                 if min_row <= row_id:
-                    yield tuple(self._get_row(element, min_col, max_col))
+                    yield tuple(self._get_row(element, min_col, max_col, row_counter=row_counter))
                     row_counter += 1
 
-            if element.tag in CELL_TAGS:
-                # sub-elements of rows should be skipped as handled within a cell
-                continue
-            element.clear()
+                element.clear()
 
 
-    def _get_row(self, element, min_col=1, max_col=None):
+    def _get_row(self, element, min_col=1, max_col=None, row_counter=None):
         """Return cells from a particular row"""
         col_counter = min_col
         data_only = getattr(self.parent, 'data_only', False)
 
         for cell in safe_iterator(element, CELL_TAG):
             coordinate = cell.get('r')
-            row, column = coordinate_to_tuple(coordinate)
+            if coordinate:
+                row, column = coordinate_to_tuple(coordinate)
+            else:
+                row, column = row_counter, col_counter
 
             if max_col is not None and column > max_col:
                 break
@@ -160,6 +166,12 @@ class ReadOnlyWorksheet(Worksheet):
                     data_type = 'f'
                     value = "=%s" % formula
 
+                elif data_type == 'inlineStr':
+                    child = cell.find(INLINE_TAG)
+                    if child is not None:
+                        richtext = Text.from_tree(child)
+                        value = richtext.content
+
                 else:
                     value = cell.findtext(VALUE_TAG) or None
 
@@ -174,23 +186,17 @@ class ReadOnlyWorksheet(Worksheet):
 
     def _get_cell(self, row, column):
         """Cells are returned by a generator which can be empty"""
-        cell = tuple(self.get_squared_range(column, row, column, row))[0]
-        if cell:
-            return cell[0]
+        for row in self.get_squared_range(column, row, column, row):
+            if row:
+                return row[0]
         return EMPTY_CELL
 
+
     @property
     def rows(self):
         return self.iter_rows()
 
 
-    @property
-    def columns(self):
-        if self.max_column is None:
-            self.calculate_dimension()
-        return super(ReadOnlyWorksheet, self).columns
-
-
     def calculate_dimension(self, force=False):
         if not all([self.max_column, self.max_row]):
             if force:
@@ -208,8 +214,11 @@ class ReadOnlyWorksheet(Worksheet):
         Loop through all the cells to get the size of a worksheet.
         Do this only if it is explicitly requested.
         """
+
         max_col = 0
         for r in self.rows:
+            if not r:
+                continue
             cell = r[-1]
             max_col = max(max_col, cell.column)
 
@@ -252,8 +261,3 @@ class ReadOnlyWorksheet(Worksheet):
     @max_column.setter
     def max_column(self, value):
         self._max_column = value
-
-
-setattr(ReadOnlyWorksheet, '__setitem__', removed_method)
-setattr(ReadOnlyWorksheet, 'range', removed_method)
-setattr(ReadOnlyWorksheet, 'merge_cells', removed_method)
